@@ -12,7 +12,7 @@
 #'   \item \emph{\code{categorical}}: Use a categorical color scale based on the values of "group.by".
 #'   \item \emph{\code{continuous}}: Use a continuous color scale based on the values of "feature".
 #' }
-#' @param order_by_mean \strong{\code{\link[base]{logical}}} | Whether to order the groups by the mean of the data (highest to lowest).
+#' @param order \strong{\code{\link[base]{logical}}} | Whether to order the groups by the median of the data (highest to lowest).
 #' @param jitter \strong{\code{\link[base]{numeric}}} | Amount of jitter in the plot along the X axis. The lower the value, the more compacted the dots are.
 #' @param colors.use \strong{\code{\link[base]{character}}} | Named vector of colors to use. Has to match the unique values of group.by when scale_type is set to categorical.
 #'
@@ -28,7 +28,7 @@ do_GeyserPlot <- function(sample,
                           split.by = NULL,
                           enforce_symmetry = FALSE,
                           scale_type = "continuous",
-                          order_by_mean = TRUE,
+                          order = TRUE,
                           plot_cell_borders = TRUE,
                           jitter = 0.45,
                           pt.size = 1,
@@ -44,9 +44,9 @@ do_GeyserPlot <- function(sample,
                           legend.type = "colorbar",
                           font.size = 14,
                           font.type = "sans",
-                          rotate_x_axis_labels = 45,
-                          viridis_color_map = "G",
-                          viridis_direction = 1,
+                          axis.text.x.angle = 45,
+                          viridis.palette = "G",
+                          viridis.direction = 1,
                           colors.use = NULL,
                           na.value = "grey75",
                           legend.ncol = NULL,
@@ -60,18 +60,34 @@ do_GeyserPlot <- function(sample,
                           xlab = "Groups",
                           ylab = feature,
                           flip = FALSE,
-                          min.cutoff = NULL,
-                          max.cutoff = NULL){
-
+                          min.cutoff = rep(NA, length(features)),
+                          max.cutoff = rep(NA, length(features)),
+                          number.breaks = 5,
+                          diverging.palette = "RdBu",
+                          diverging.direction = -1,
+                          sequential.palette = "YlGnBu",
+                          sequential.direction = -1,
+                          use_viridis = TRUE,
+                          plot.title.face = "bold",
+                          plot.subtitle.face = "plain",
+                          plot.caption.face = "italic",
+                          axis.title.face = "bold",
+                          axis.text.face = "plain",
+                          legend.title.face = "bold",
+                          legend.text.face = "plain"){
+  # Add lengthy error messages.
+  withr::local_options(.new = list("warning.length" = 8170))
+  
   check_suggests(function_name = "do_GeyserPlot")
   # Check if the sample provided is a Seurat object.
   check_Seurat(sample = sample)
 
   # Check logical parameters.
   logical_list <- list("enforce_symmetry" = enforce_symmetry,
-                       "order_by_mean" = order_by_mean,
+                       "order" = order,
                        "plot_cell_borders" = plot_cell_borders,
-                       "flip" = flip)
+                       "flip" = flip,
+                       "use_viridis" = use_viridis)
   check_type(parameters = logical_list, required_type = "logical", test_function = is.logical)
   # Check numeric parameters.
   numeric_list <- list("pt.size" = pt.size,
@@ -85,8 +101,10 @@ do_GeyserPlot <- function(sample,
                        "legend.ncol" = legend.ncol,
                        "legend.nrow" = legend.nrow,
                        "legend.icon.size" = legend.icon.size,
-                       "viridis_direction" = viridis_direction,
-                       "rotate_x_axis_labels" = rotate_x_axis_labels)
+                       "viridis.direction" = viridis.direction,
+                       "axis.text.x.angle" = axis.text.x.angle,
+                       "number.breaks" = number.breaks,
+                       "sequential.direction" = sequential.direction)
   check_type(parameters = numeric_list, required_type = "numeric", test_function = is.numeric)
   # Check character parameters.
 
@@ -100,13 +118,22 @@ do_GeyserPlot <- function(sample,
                          "plot.subtitle" = plot.subtitle,
                          "plot.caption" = plot.caption,
                          "scale_type" = scale_type,
-                         "viridis_color_map" = viridis_color_map,
+                         "viridis.palette" = viridis.palette,
                          "legend.framecolor" = legend.framecolor,
                          "legend.tickcolor" = legend.tickcolor,
                          "legend.type" = legend.type,
                          "font.type" = font.type,
                          "border.color" = border.color,
-                         "na.value" = na.value)
+                         "na.value" = na.value,
+                         "diverging.palette" = diverging.palette,
+                         "sequential.palette" = sequential.palette,
+                         "plot.title.face" = plot.title.face,
+                         "plot.subtitle.face" = plot.subtitle.face,
+                         "plot.caption.face" = plot.caption.face,
+                         "axis.title.face" = axis.title.face,
+                         "axis.text.face" = axis.text.face,
+                         "legend.title.face" = legend.title.face,
+                         "legend.text.face" = legend.text.face)
   # Checks
   check_type(parameters = character_list, required_type = "character", test_function = is.character)
 
@@ -118,12 +145,37 @@ do_GeyserPlot <- function(sample,
   check_parameters(parameter = font.type, parameter_name = "font.type")
   check_parameters(parameter = legend.type, parameter_name = "legend.type")
   check_parameters(parameter = legend.position, parameter_name = "legend.position")
-  check_parameters(parameter = viridis_direction, parameter_name = "viridis_direction")
-  check_parameters(parameter = viridis_color_map, parameter_name = "viridis_color_map")
+  check_parameters(parameter = viridis.palette, parameter_name = "viridis.palette")
   check_parameters(parameter = scale_type, parameter_name = "scale_type")
-  check_parameters(parameter = rotate_x_axis_labels, parameter_name = "rotate_x_axis_labels")
-
+  check_parameters(parameter = axis.text.x.angle, parameter_name = "axis.text.x.angle")
+  check_parameters(parameter = number.breaks, parameter_name = "number.breaks")
+  check_parameters(parameter = diverging.palette, parameter_name = "diverging.palette")
+  check_parameters(plot.title.face, parameter_name = "plot.title.face")
+  check_parameters(plot.subtitle.face, parameter_name = "plot.subtitle.face")
+  check_parameters(plot.caption.face, parameter_name = "plot.caption.face")
+  check_parameters(axis.title.face, parameter_name = "axis.title.face")
+  check_parameters(axis.text.face, parameter_name = "axis.text.face")
+  check_parameters(legend.title.face, parameter_name = "legend.title.face")
+  check_parameters(legend.text.face, parameter_name = "legend.text.face")
+  check_parameters(viridis.direction, parameter_name = "viridis.direction")
+  check_parameters(sequential.direction, parameter_name = "sequential.direction")
+  check_parameters(diverging.direction, parameter_name = "diverging.direction")
+  
   `%>%` <- magrittr::`%>%`
+  
+  # Generate the continuous color palette.
+  if (isTRUE(enforce_symmetry)){
+    colors.gradient <- compute_continuous_palette(name = diverging.palette,
+                                                  use_viridis = FALSE,
+                                                  direction = diverging.direction,
+                                                  enforce_symmetry = enforce_symmetry)
+  } else {
+    colors.gradient <- compute_continuous_palette(name = ifelse(isTRUE(use_viridis), viridis.palette, sequential.palette),
+                                                  use_viridis = use_viridis,
+                                                  direction = ifelse(isTRUE(use_viridis), viridis.direction, sequential.direction),
+                                                  enforce_symmetry = enforce_symmetry)
+  }
+  
   # Check the assay.
   out <- check_and_set_assay(sample = sample, assay = assay)
   sample <- out[["sample"]]
@@ -133,41 +185,95 @@ do_GeyserPlot <- function(sample,
   # Check that split.by is in metadata variables.
   if (!is.null(split.by)){
     assertthat::assert_that(split.by %in% colnames(sample@meta.data),
-                            msg = "The variable for split.by has to be on the metadata of the object.")
+                            msg = paste0(add_cross(), crayon_body("The variable for "),
+                                         crayon_key("split.by"),
+                                         crayon_body(" has to be on the "),
+                                         crayon_key("metadata"),
+                                         crayon_body(" of the Seurat object.")))
   }
 
 
   # Check that group.by is in metadata variables.
   if (!is.null(group.by)){
     assertthat::assert_that(group.by %in% colnames(sample@meta.data),
-                            msg = "The variable for group.by has to be on the metadata of the object.")
+                            msg = paste0(add_cross(), crayon_body("The variable for "),
+                                         crayon_key("group.by"),
+                                         crayon_body(" has to be on the "),
+                                         crayon_key("metadata"),
+                                         crayon_body(" of the Seurat object.")))
   }
 
   # Check that jitter is in range.
   assertthat::assert_that(jitter > 0 & jitter < 0.5,
-                          msg = "Value for jitter has to be betwen 0 and 0.49.")
+                          msg = paste0(add_cross(), crayon_body("The value for "),
+                                       crayon_key("jitter"),
+                                       crayon_body(" has to be between "),
+                                       crayon_key("0"),
+                                       crayon_body(" and "),
+                                       crayon_key("0.49"),
+                                       crayon_body(".")))
+
+  assertthat::assert_that(length(min.cutoff) == length(features),
+                          msg = paste0(add_cross(), crayon_body("Please provide "),
+                                       crayon_key("as many values"),
+                                       crayon_body(" to "),
+                                       crayon_key("min.cutoff"),
+                                       crayon_body(" than provided "),
+                                       crayon_key("features"),
+                                       crayon_body(". Use"),
+                                       crayon_key("NA"),
+                                       crayon_body(" if you want to skip a given feature.")))
+
+  assertthat::assert_that(length(max.cutoff) == length(features),
+                          msg = paste0(add_cross(), crayon_body("Please provide "),
+                                       crayon_key("as many values"),
+                                       crayon_body(" to "),
+                                       crayon_key("max.cutoff"),
+                                       crayon_body(" than provided "),
+                                       crayon_key("features"),
+                                       crayon_body(". Use"),
+                                       crayon_key("NA"),
+                                       crayon_body(" if you want to skip a given feature.")))
 
   # Will contain the output.
   list.out <- list()
 
-  # Assign group.by to a metadata variable.
-  if (is.null(group.by)){
-    sample@meta.data[, "Groups"] <- sample@active.ident
-    group.by <- "Groups"
+  # Check group.by.
+  out <- check_group_by(sample = sample,
+                        group.by = group.by,
+                        is.heatmap = FALSE)
+  sample <- out[["sample"]]
+  group.by <- out[["group.by"]]
+  
+  if (is.null(colors.use)){
+    colors.use <- generate_color_scale(names_use = if (is.factor(sample@meta.data[, group.by])) {
+      levels(sample@meta.data[, group.by])
+    } else {
+      sort(unique(sample@meta.data[, group.by]))
+    })
+  } else {
+    check_colors(colors.use)
+    check_consistency_colors_and_names(sample, colors = colors.use, grouping_variable = group.by)
   }
 
   # Iterate for each feature.
+  counter <- 0
   for (feature in features){
+    counter <- counter + 1
+
+    max.cutoff.use <- max.cutoff[counter]
+    min.cutoff.use <- min.cutoff[counter]
+
     # Check the feature.
     check_feature(sample = sample,
                   features = feature)
 
 
-    # Get a vector of all dimensional reduction compontents.
-    dim_colnames <- c()
+    # Get a vector of all dimensional reduction components.
+    dim_colnames <- NULL
     for(red in Seurat::Reductions(object = sample)){
       col.names <- colnames(sample@reductions[[red]][[]])
-      dim_colnames <- c(dim_colnames, col.names)
+      dim_colnames <- append(dim_colnames, col.names)
       if (feature %in% col.names){
         # Get the reduction in which the feature is, if this is the case.
         reduction <- red
@@ -182,7 +288,7 @@ do_GeyserPlot <- function(sample,
               tibble::rownames_to_column(var = "cell") %>%
               tibble::as_tibble()
     } else if (isTRUE(feature %in% rownames(sample))){
-      data <- Seurat::GetAssayData(object = sample,
+      data <- .GetAssayData(sample = sample,
                                    assay = assay,
                                    slot = slot)[feature, , drop = FALSE] %>%
               as.matrix() %>%
@@ -218,16 +324,16 @@ do_GeyserPlot <- function(sample,
     }
 
     # Proceed with the regular plot.
-    if (isTRUE(order_by_mean)){
+    if (isTRUE(order)){
       data <- data %>%
               dplyr::mutate("group.by" = factor(.data[[group.by]], levels = {data %>%
                                                                              dplyr::group_by(.data[[group.by]]) %>%
-                                                                             dplyr::summarise("mean" = mean(.data[[feature]])) %>%
+                                                                             dplyr::summarise("mean" = stats::median(.data[[feature]]), na.rm = TRUE) %>%
                                                                              dplyr::arrange(dplyr::desc(.data$mean)) %>%
                                                                              dplyr::pull(.data[[group.by]]) %>%
                                                                              as.character()}),
                             "values" = .data[[feature]])
-    } else if (isFALSE(order_by_mean)){
+    } else if (base::isFALSE(order)){
       data <- data %>%
               dplyr::mutate("group.by" = .data[[group.by]],
                             "values" = .data[[feature]])
@@ -243,25 +349,15 @@ do_GeyserPlot <- function(sample,
             dplyr::select(dplyr::all_of(cols.use))
 
     # Define cutoffs.
-    range.data <- c(min(data[, "values"], na.rm = TRUE), max(data[, "values"], na.rm = TRUE))
+    range.data <- c(min(data[, "values"], na.rm = TRUE),
+                    max(data[, "values"], na.rm = TRUE))
 
+    out <- check_cutoffs(min.cutoff = min.cutoff.use,
+                         max.cutoff = max.cutoff.use,
+                         feature = feature,
+                         limits = range.data)
 
-    if (!is.null(min.cutoff) & !is.null(max.cutoff)){
-      assertthat::assert_that(min.cutoff < max.cutoff,
-                              msg = paste0("The value provided for min.cutoff (", min.cutoff, ") has to be lower than the value provided to max.cutoff (", max.cutoff, "). Please select another value."))
-    }
-
-    if (!is.null(min.cutoff)){
-      assertthat::assert_that(min.cutoff >= range.data[1],
-                              msg = paste0("The value provided for min.cutoff (", min.cutoff, ") is lower than the minimum value in the enrichment matrix (", range.data[1], "). Please select another value."))
-      range.data <- c(min.cutoff, range.data[2])
-    }
-
-    if (!is.null(max.cutoff)){
-      assertthat::assert_that(max.cutoff <= range.data[2],
-                              msg = paste0("The value provided for max.cutoff (", max.cutoff, ") is lower than the maximum value in the enrichment matrix (", range.data[2], "). Please select another value."))
-      range.data <- c(range.data[1], max.cutoff)
-    }
+    range.data <- out$limits
 
 
     # Plot.
@@ -287,54 +383,42 @@ do_GeyserPlot <- function(sample,
                                na.rm = TRUE)
     }
 
+    scale.setup <- compute_scales(sample = sample,
+                                  feature = feature,
+                                  assay = assay,
+                                  reduction = NULL,
+                                  slot = slot,
+                                  number.breaks = number.breaks,
+                                  min.cutoff = min.cutoff.use,
+                                  max.cutoff = max.cutoff.use,
+                                  flavor = "Seurat",
+                                  enforce_symmetry = enforce_symmetry,
+                                  from_data = TRUE,
+                                  limits.use = range.data)
+
+    limits <- c(min(data[, "values"], na.rm = TRUE),
+                max(data[, "values"], na.rm = TRUE))
+
+    if (limits[1] != range.data[1]){
+      limits <- c(range.data[1], limits[2])
+    }
+
+    if (limits[2] != range.data[2]){
+      limits <- c(limits[1], range.data[2])
+    }
+    end_value <- max(abs(limits))
     if (isTRUE(scale_type == "continuous")){
-      if (isTRUE(enforce_symmetry)){
-        limits <- c(min(data[, "values"], na.rm = TRUE),
-                    max(data[, "values"], na.rm = TRUE))
-        if (limits[1] != range.data[1]){
-          limits <- c(range.data[1], limits[2])
-        }
-
-        if (limits[2] != range.data[2]){
-          limits <- c(limits[1], range.data[2])
-        }
-        end_value <- max(abs(limits))
-        scale.use <- ggplot2::scale_color_gradientn(colors = c("#033270", "#4091C9", "grey95", "#c94040", "#65010C"),
-                                                    limits = c(-end_value, end_value),
-                                                    na.value = na.value)
-
-      } else if (isFALSE(enforce_symmetry)){
-        scale.use <- ggplot2::scale_color_viridis_c(option = viridis_color_map,
-                                                    na.value = na.value,
-                                                    direction = viridis_direction,
-                                                    limits = range.data)
-      }
+      p <- p + 
+           ggplot2::scale_color_gradientn(colors = colors.gradient,
+                                          na.value = na.value,
+                                          name = legend.title,
+                                          breaks = scale.setup$breaks,
+                                          labels = scale.setup$labels,
+                                          limits = scale.setup$limits)
     } else if (isTRUE(scale_type == "categorical")){
-      limits <- c(min(data[, "values"], na.rm = TRUE),
-                  max(data[, "values"], na.rm = TRUE))
-      if (limits[1] != range.data[1]){
-        limits <- c(range.data[1], limits[2])
-      }
-
-      if (limits[2] != range.data[2]){
-        limits <- c(limits[1], range.data[2])
-      }
-      end_value <- max(abs(limits))
-      if (is.null(colors.use)){
-        values <- data %>% dplyr::pull(.data[["group.by"]])
-        if (is.factor(values)){
-          names.use <- levels(values)
-        } else {
-          # nocov start
-          sort(unique(values))
-          # nocov end
-        }
-        colors.use <- generate_color_scale(names_use = names.use)
-      } else {
-        check_colors(colors.use)
-      }
-      scale.use <- ggplot2::scale_color_manual(values = colors.use,
-                                               na.value = na.value)
+      p <- p + 
+           ggplot2::scale_color_manual(values = colors.use,
+                                       na.value = na.value)
     }
 
     p <- p +
@@ -354,8 +438,7 @@ do_GeyserPlot <- function(sample,
                                     point_color = "black",
                                     position = ggplot2::position_dodge(width = 1),
                                     na.rm = TRUE,
-                                    show.legend = FALSE) +
-         scale.use
+                                    show.legend = FALSE)
 
     if (!(is.null(split.by))){
       p <- p +
@@ -371,27 +454,27 @@ do_GeyserPlot <- function(sample,
          ggplot2::ylab(ylab) +
          ggplot2::theme_minimal(base_size = font.size) +
          ggplot2::theme(axis.title = ggplot2::element_text(color = "black",
-                                                           face = "bold"),
-                        axis.line.x = if (isFALSE(flip)) {ggplot2::element_line(color = "black")} else if (isTRUE(flip)) {ggplot2::element_blank()},
-                        axis.line.y = if (isTRUE(flip)) {ggplot2::element_line(color = "black")} else if (isFALSE(flip)) {ggplot2::element_blank()},
+                                                           face = axis.title.face),
+                        axis.line.x = if (base::isFALSE(flip)) {ggplot2::element_line(color = "black")} else if (isTRUE(flip)) {ggplot2::element_blank()},
+                        axis.line.y = if (isTRUE(flip)) {ggplot2::element_line(color = "black")} else if (base::isFALSE(flip)) {ggplot2::element_blank()},
                         axis.text.x = ggplot2::element_text(color = "black",
-                                                            face = "bold",
-                                                            angle = get_axis_parameters(angle = rotate_x_axis_labels, flip = flip)[["angle"]],
-                                                            hjust = get_axis_parameters(angle = rotate_x_axis_labels, flip = flip)[["hjust"]],
-                                                            vjust = get_axis_parameters(angle = rotate_x_axis_labels, flip = flip)[["vjust"]]),
-                        axis.text.y = ggplot2::element_text(color = "black", face = "bold"),
+                                                            face = axis.text.face,
+                                                            angle = get_axis_parameters(angle = axis.text.x.angle, flip = flip)[["angle"]],
+                                                            hjust = get_axis_parameters(angle = axis.text.x.angle, flip = flip)[["hjust"]],
+                                                            vjust = get_axis_parameters(angle = axis.text.x.angle, flip = flip)[["vjust"]]),
+                        axis.text.y = ggplot2::element_text(color = "black", face = axis.text.face),
                         axis.ticks = ggplot2::element_line(color = "black"),
                         panel.grid.major = ggplot2::element_blank(),
                         plot.title.position = "plot",
-                        plot.title = ggplot2::element_text(face = "bold", hjust = 0),
-                        plot.subtitle = ggplot2::element_text(hjust = 0),
-                        plot.caption = ggplot2::element_text(hjust = 1),
+                        plot.title = ggplot2::element_text(face = plot.title.face, hjust = 0),
+                        plot.subtitle = ggplot2::element_text(face = plot.subtitle.face, hjust = 0),
+                        plot.caption = ggplot2::element_text(face = plot.caption.face, hjust = 1),
+                        legend.text = ggplot2::element_text(face = legend.text.face),
+                        legend.title = ggplot2::element_text(face = legend.title.face),
                         panel.grid = ggplot2::element_blank(),
                         text = ggplot2::element_text(family = font.type),
                         plot.caption.position = "plot",
-                        legend.text = ggplot2::element_text(face = "bold"),
                         legend.position = legend.position,
-                        legend.title = ggplot2::element_text(face = "bold"),
                         legend.justification = "center",
                         plot.margin = ggplot2::margin(t = 10, r = 10, b = 10, l = 10),
                         plot.background = ggplot2::element_rect(fill = "white", color = "white"),
