@@ -37,7 +37,7 @@ do_RidgePlot <- function(sample,
                          colors.use = NULL,
                          font.size = 14,
                          font.type = "sans",
-                         rotate_x_axis_labels = 45,
+                         axis.text.x.angle = 45,
                          plot.title = NULL,
                          plot.subtitle = NULL,
                          plot.caption = NULL,
@@ -49,12 +49,26 @@ do_RidgePlot <- function(sample,
                          compute_distribution_tails = FALSE,
                          prob_tails = 0.025,
                          color_by_probabilities = FALSE,
-                         viridis_color_map = "G",
-                         viridis_direction = 1,
+                         use_viridis = TRUE,
+                         viridis.palette = "G",
+                         viridis.direction = 1,
+                         sequential.palette = "YlGnBu",
+                         sequential.direction = 1,
                          plot.grid = TRUE,
                          grid.color = "grey75",
                          grid.type = "dashed",
-                         flip = FALSE){
+                         flip = FALSE,
+                         number.breaks = 5,
+                         plot.title.face = "bold",
+                         plot.subtitle.face = "plain",
+                         plot.caption.face = "italic",
+                         axis.title.face = "bold",
+                         axis.text.face = "plain",
+                         legend.title.face = "bold",
+                         legend.text.face = "plain"){
+  # Add lengthy error messages.
+  withr::local_options(.new = list("warning.length" = 8170))
+  
   check_suggests(function_name = "do_RidgePlot")
   `%>%` <- magrittr::`%>%`
 
@@ -69,7 +83,8 @@ do_RidgePlot <- function(sample,
                        "color_by_probabilities" = color_by_probabilities,
                        "plot.grid" = plot.grid,
                        "flip" = flip,
-                       "legend.nrow" = legend.nrow)
+                       "legend.nrow" = legend.nrow,
+                       "use_viridis" = use_viridis)
   check_type(parameters = logical_list, required_type = "logical", test_function = is.logical)
   # Check numeric parameters.
   numeric_list <- list("legend.width" = legend.width,
@@ -79,10 +94,12 @@ do_RidgePlot <- function(sample,
                        "font.size" = font.size,
                        "quantiles" = quantiles,
                        "prob_tails" = prob_tails,
-                       "viridis_direction" = viridis_direction,
-                       "rotate_x_axis_labels" = rotate_x_axis_labels,
+                       "viridis.direction" = viridis.direction,
+                       "axis.text.x.angle" = axis.text.x.angle,
                        "legend.ncol" = legend.ncol,
-                       "legend.nrow" = legend.nrow)
+                       "legend.nrow" = legend.nrow,
+                       "number.breaks" = number.breaks,
+                       "sequential.direction" = sequential.direction)
   check_type(parameters = numeric_list, required_type = "numeric", test_function = is.numeric)
   # Check character parameters.
   character_list <- list("feature" = feature,
@@ -102,9 +119,18 @@ do_RidgePlot <- function(sample,
                          "plot.caption" = plot.caption,
                          "xlab" = xlab,
                          "ylab" = ylab,
-                         "viridis_color_map" = viridis_color_map,
+                         "viridis.palette" = viridis.palette,
                          "grid.color" = grid.color,
-                         "grid.type" = grid.type)
+                         "grid.type" = grid.type,
+                         "sequential.palette" = sequential.palette,
+                         "plot.title.face" = plot.title.face,
+                         "plot.subtitle.face" = plot.subtitle.face,
+                         "plot.caption.face" = plot.caption.face,
+                         "axis.title.face" = axis.title.face,
+                         "axis.text.face" = axis.text.face,
+                         "legend.title.face" = legend.title.face,
+                         "legend.text.face" = legend.text.face)
+  check_type(parameters = character_list, required_type = "character", test_function = is.character)
 
   check_colors(legend.tickcolor, parameter_name = "legend.tickcolor")
   check_colors(legend.framecolor, parameter_name = "legend.framecolor")
@@ -112,17 +138,38 @@ do_RidgePlot <- function(sample,
   check_parameters(parameter = font.type, parameter_name = "font.type")
   check_parameters(parameter = legend.type, parameter_name = "legend.type")
   if (!is.null(legend.position)){check_parameters(parameter = legend.position, parameter_name = "legend.position")}
-  check_parameters(parameter = viridis_direction, parameter_name = "viridis_direction")
-  check_parameters(parameter = viridis_color_map, parameter_name = "viridis_color_map")
+  check_parameters(parameter = viridis.palette, parameter_name = "viridis.palette")
   check_parameters(parameter = grid.type, parameter_name = "grid.type")
-  check_parameters(parameter = rotate_x_axis_labels, parameter_name = "rotate_x_axis_labels")
-
+  check_parameters(parameter = axis.text.x.angle, parameter_name = "axis.text.x.angle")
+  check_parameters(parameter = number.breaks, parameter_name = "number.breaks")
+  check_parameters(plot.title.face, parameter_name = "plot.title.face")
+  check_parameters(plot.subtitle.face, parameter_name = "plot.subtitle.face")
+  check_parameters(plot.caption.face, parameter_name = "plot.caption.face")
+  check_parameters(axis.title.face, parameter_name = "axis.title.face")
+  check_parameters(axis.text.face, parameter_name = "axis.text.face")
+  check_parameters(legend.title.face, parameter_name = "legend.title.face")
+  check_parameters(legend.text.face, parameter_name = "legend.text.face")
+  check_parameters(viridis.direction, parameter_name = "viridis.direction")
+  check_parameters(sequential.direction, parameter_name = "sequential.direction")
+  
+  colors.gradient <- compute_continuous_palette(name = ifelse(isTRUE(use_viridis), viridis.palette, sequential.palette),
+                                                use_viridis = use_viridis,
+                                                direction = ifelse(isTRUE(use_viridis), viridis.direction, sequential.direction),
+                                                enforce_symmetry = FALSE)
+  
   if (!is.null(colors.use)){check_colors(colors.use, parameter_name = "colors.use")}
 
   if (is.null(legend.position)){
     legend.position <- ifelse(isTRUE(continuous_scale), "bottom", "none")
   }
-
+  
+  # Check group.by.
+  out <- check_group_by(sample = sample,
+                        group.by = group.by,
+                        is.heatmap = FALSE)
+  sample <- out[["sample"]]
+  group.by <- out[["group.by"]]
+  
   data <- get_data_column_in_context(sample = sample,
                                      feature = feature,
                                      assay = assay,
@@ -130,17 +177,17 @@ do_RidgePlot <- function(sample,
                                      group.by = group.by,
                                      split.by = split.by)
   if (isTRUE(continuous_scale)){
-    if (isFALSE(compute_quantiles)){
+    if (base::isFALSE(compute_quantiles)){
       p <- data %>%
            ggplot2::ggplot(mapping = ggplot2::aes(x = .data$feature,
                                                   y = .data$group.by,
                                                   fill = ggplot2::after_stat(x))) +
            ggridges::geom_density_ridges_gradient(color = "black",
-                                                  size = 1.25) +
-           ggplot2::scale_fill_viridis_c(option = viridis_color_map,
-                                         direction = viridis_direction,
-                                         name = feature)
-
+                                                  size = 1.25) + 
+           ggplot2::scale_fill_gradientn(colors = colors.gradient,
+                                         na.value = "grey75",
+                                         name = feature,
+                                         breaks = scales::extended_breaks(n = number.breaks))
       p <- modify_continuous_legend(p = p,
                                     legend.aes = "fill",
                                     legend.type = legend.type,
@@ -153,15 +200,15 @@ do_RidgePlot <- function(sample,
                                     legend.tickwidth = legend.tickwidth)
     } else if (isTRUE(compute_quantiles)){
       if (isTRUE(compute_custom_quantiles)){
-        labels <- c()
+        labels <- NULL
         for (i in seq_along(quantiles)){
           if (i == 1){
-            labels <- c(labels, paste0("[0 , ", quantiles[i], "["))
+            labels <- append(labels, paste0("[0 , ", quantiles[i], "["))
           } else if (i == length(quantiles)){
-            labels <- c(labels, paste0("]", quantiles[i], ", 1]"))
+            labels <- append(labels, paste0("]", quantiles[i], ", 1]"))
           } else {
-            labels <- c(labels, paste0("]", quantiles[i - 1], ", ", quantiles[i], "]"))
-            labels <- c(labels, paste0("]", quantiles[i], ", ", quantiles[i + 1], "]"))
+            labels <- append(labels, paste0("]", quantiles[i - 1], ", ", quantiles[i], "]"))
+            labels <- append(labels, paste0("]", quantiles[i], ", ", quantiles[i + 1], "]"))
           }
         }
         p <- data %>%
@@ -173,10 +220,19 @@ do_RidgePlot <- function(sample,
                                            quantile_lines = TRUE,
                                            calc_ecdf = TRUE,
                                            geom = "density_ridges_gradient",
-                                           quantiles = quantiles) +
-             ggplot2::scale_fill_manual(values = viridis::viridis(n = length(quantiles) + 1, option = viridis_color_map, direction = viridis_direction),
-                                        name = ifelse(is.null(legend.title), "Probability", legend.title),
-                                        labels = unique(labels)) +
+                                           quantiles = quantiles)
+        if (isTRUE(use_viridis)){
+          p  <- p + 
+                ggplot2::scale_fill_manual(values = viridis::viridis(n = length(quantiles) + 1, option = viridis.palette, direction = viridis.direction),
+                                           name = ifelse(is.null(legend.title), "Probability", legend.title),
+                                           labels = unique(labels))
+        } else {
+          p <- p + 
+               ggplot2::scale_fill_manual(values = if (sequential.direction == 1) {RColorBrewer::brewer.pal(n = length(quantiles) + 1, name = sequential.palette)} else {rev(RColorBrewer::brewer.pal(n = length(quantiles) + 1, name = sequential.palette))},
+                                          name = ifelse(is.null(legend.title), "Probability", legend.title),
+                                          labels = unique(labels))
+        }
+        p <- p +
              ggplot2::guides(fill = ggplot2::guide_legend(title = ifelse(is.null(legend.title), "Probability", legend.title),
                                                           title.position = "top",
                                                           title.hjust = 0.5,
@@ -211,10 +267,12 @@ do_RidgePlot <- function(sample,
              ggridges::stat_density_ridges(color = "black",
                                            size = 1.25,
                                            calc_ecdf = TRUE,
-                                           geom = "density_ridges_gradient") +
-             ggplot2::scale_fill_viridis_c(option = viridis_color_map,
+                                           geom = "density_ridges_gradient") + 
+             ggplot2::scale_fill_gradientn(colors = colors.gradient,
+                                           na.value = "grey75",
                                            name = "Tail probability",
-                                           direction = viridis_direction)
+                                           breaks = scales::extended_breaks(n = number.breaks))
+
         p <- modify_continuous_legend(p = p,
                                       legend.title = legend.title,
                                       legend.aes = "fill",
@@ -229,7 +287,7 @@ do_RidgePlot <- function(sample,
       }
     }
 
-  } else if (isFALSE(continuous_scale)){
+  } else if (base::isFALSE(continuous_scale)){
     p <- data %>%
          ggplot2::ggplot(mapping = ggplot2::aes(x = .data$feature,
                                                 y = .data$group.by,
@@ -247,7 +305,7 @@ do_RidgePlot <- function(sample,
   if (!is.null(split.by)){
     # Facet.
     p <- p +
-         ggplot2::facet_grid( ~ .data$split.by)
+         ggplot2::facet_grid(~ .data$split.by)
   }
 
 
@@ -259,29 +317,29 @@ do_RidgePlot <- function(sample,
        ggplot2::ylab(if (is.null(ylab)) {"Groups"} else (ylab)) +
        ggplot2::theme_minimal(base_size = font.size) +
        ggplot2::theme(axis.title = ggplot2::element_text(color = "black",
-                                                         face = "bold"),
-                      axis.line.y = if (isFALSE(flip)) {ggplot2::element_line(color = "black")} else if (isTRUE(flip)) {ggplot2::element_blank()},
-                      axis.line.x = if (isTRUE(flip)) {ggplot2::element_line(color = "black")} else if (isFALSE(flip)) {ggplot2::element_blank()},
+                                                         face = axis.title.face),
+                      axis.line.y = if (base::isFALSE(flip)) {ggplot2::element_line(color = "black")} else if (isTRUE(flip)) {ggplot2::element_blank()},
+                      axis.line.x = if (isTRUE(flip)) {ggplot2::element_line(color = "black")} else if (base::isFALSE(flip)) {ggplot2::element_blank()},
                       axis.text.x = ggplot2::element_text(color = "black",
-                                                          face = "bold",
-                                                          angle = get_axis_parameters(angle = rotate_x_axis_labels, flip = flip)[["angle"]],
-                                                          hjust = get_axis_parameters(angle = rotate_x_axis_labels, flip = flip)[["hjust"]],
-                                                          vjust = get_axis_parameters(angle = rotate_x_axis_labels, flip = flip)[["vjust"]]),
-                      axis.text.y = ggplot2::element_text(color = "black", face = "bold"),
+                                                          face = axis.text.face,
+                                                          angle = get_axis_parameters(angle = axis.text.x.angle, flip = flip)[["angle"]],
+                                                          hjust = get_axis_parameters(angle = axis.text.x.angle, flip = flip)[["hjust"]],
+                                                          vjust = get_axis_parameters(angle = axis.text.x.angle, flip = flip)[["vjust"]]),
+                      axis.text.y = ggplot2::element_text(color = "black", face = axis.text.face),
                       axis.ticks = ggplot2::element_line(color = "black"),
                       panel.grid.major = ggplot2::element_blank(),
                       panel.grid.minor = ggplot2::element_blank(),
                       plot.title.position = "plot",
-                      plot.title = ggplot2::element_text(face = "bold", hjust = 0),
-                      plot.subtitle = ggplot2::element_text(hjust = 0),
-                      plot.caption = ggplot2::element_text(hjust = 1),
+                      plot.title = ggplot2::element_text(face = plot.title.face, hjust = 0),
+                      plot.subtitle = ggplot2::element_text(face = plot.subtitle.face, hjust = 0),
+                      plot.caption = ggplot2::element_text(face = plot.caption.face, hjust = 1),
+                      legend.text = ggplot2::element_text(face = legend.text.face),
+                      legend.title = ggplot2::element_text(face = legend.title.face),
                       panel.grid.major.y = ggplot2::element_blank(),
                       panel.grid.major.x = if (isTRUE(plot.grid)){ggplot2::element_line(color = grid.color, linetype = grid.type)} else {ggplot2::element_blank()},
                       text = ggplot2::element_text(family = font.type),
                       plot.caption.position = "plot",
-                      legend.text = ggplot2::element_text(face = "bold"),
                       legend.position = legend.position,
-                      legend.title = ggplot2::element_text(face = "bold"),
                       legend.justification = "center",
                       plot.margin = ggplot2::margin(t = 10, r = 10, b = 10, l = 10),
                       plot.background = ggplot2::element_rect(fill = "white", color = "white"),
