@@ -36,7 +36,17 @@ do_VolcanoPlot <- function(sample,
                            order_tags_by = "both",
                            n_genes = 5,
                            use_labels = FALSE,
-                           colors.use = "steelblue"){
+                           colors.use = "steelblue",
+                           plot.title.face = "bold",
+                           plot.subtitle.face = "plain",
+                           plot.caption.face = "italic",
+                           axis.title.face = "bold",
+                           axis.text.face = "plain",
+                           legend.title.face = "bold",
+                           legend.text.face = "plain"){
+  # Add lengthy error messages.
+  withr::local_options(.new = list("warning.length" = 8170))
+  
   check_suggests(function_name = "do_VolcanoPlot")
   # Check if the sample provided is a Seurat object.
   check_Seurat(sample = sample)
@@ -63,7 +73,14 @@ do_VolcanoPlot <- function(sample,
                          "plot.subtitle" = plot.subtitle,
                          "plot.caption" = plot.caption,
                          "order_tags_by" = order_tags_by,
-                         "colors.use" = colors.use)
+                         "colors.use" = colors.use,
+                         "plot.title.face" = plot.title.face,
+                         "plot.subtitle.face" = plot.subtitle.face,
+                         "plot.caption.face" = plot.caption.face,
+                         "axis.title.face" = axis.title.face,
+                         "axis.text.face" = axis.text.face,
+                         "legend.title.face" = legend.title.face,
+                         "legend.text.face" = legend.text.face)
   check_type(parameters = character_list, required_type = "character", test_function = is.character)
 
   check_colors(border.color, parameter_name = "border.color")
@@ -71,13 +88,20 @@ do_VolcanoPlot <- function(sample,
   check_colors(colors.use, parameter_name = "colors.use")
 
   check_parameters(parameter = font.type, parameter_name = "font.type")
-
+  check_parameters(plot.title.face, parameter_name = "plot.title.face")
+  check_parameters(plot.subtitle.face, parameter_name = "plot.subtitle.face")
+  check_parameters(plot.caption.face, parameter_name = "plot.caption.face")
+  check_parameters(axis.title.face, parameter_name = "axis.title.face")
+  check_parameters(axis.text.face, parameter_name = "axis.text.face")
+  check_parameters(legend.title.face, parameter_name = "legend.title.face")
+  check_parameters(legend.text.face, parameter_name = "legend.text.face")
+  
+  
   assertthat::assert_that(order_tags_by %in% c("both", "pvalue", "logfc"),
                           msg = "Please use either both, pvalue or logfc in order_tags_by.")
 
-  `.` <- plyr::.()
   `%>%` <- magrittr::`%>%`
-  colors <- SCpubr::do_ColorPalette(colors.use, tetradic = TRUE)
+  colors <- do_ColorPalette(colors.use, tetradic = TRUE)
   names(colors) <- c("A", "C", "B", "D")
 
   if (!("gene" %in% colnames(de_genes))){
@@ -180,7 +204,7 @@ do_VolcanoPlot <- function(sample,
     }
     data.label <- dplyr::bind_rows(data.up, data.down)
 
-    if (isFALSE(use_labels)){
+    if (base::isFALSE(use_labels)){
       p <- p +
            ggrepel::geom_text_repel(data = data.label,
                                     mapping = ggplot2::aes(label = .data$gene),
@@ -200,20 +224,20 @@ do_VolcanoPlot <- function(sample,
   p <- p +
        ggplot2::theme_minimal(base_size = font.size) +
        ggplot2::theme(plot.margin = ggplot2::margin(t = 10, r = 10, b = 10, l = 10),
-                      plot.title = ggplot2::element_text(face = "bold", hjust = 0),
-                      plot.subtitle = ggplot2::element_text(hjust = 0),
-                      plot.caption = ggplot2::element_text(hjust = 1),
+                      plot.title = ggplot2::element_text(face = plot.title.face, hjust = 0),
+                      plot.subtitle = ggplot2::element_text(face = plot.subtitle.face, hjust = 0),
+                      plot.caption = ggplot2::element_text(face = plot.caption.face, hjust = 1),
+                      legend.text = ggplot2::element_text(face = legend.text.face),
+                      legend.title = ggplot2::element_text(face = legend.title.face),
                       panel.grid = ggplot2::element_blank(),
                       plot.title.position = "plot",
                       plot.caption.position = "plot",
                       text = ggplot2::element_text(family = font.type),
-                      legend.text = ggplot2::element_text(face = "bold"),
                       legend.position = "none",
-                      legend.title = ggplot2::element_text(face = "bold"),
                       legend.justification = "center",
-                      axis.title.x = ggplot2::element_text(face = "bold", color = "black"),
-                      axis.title.y = ggplot2::element_text(face = "bold", angle = 90, color = "black"),
-                      axis.text = ggplot2::element_text(face = "bold", color = "black"),
+                      axis.title.x = ggplot2::element_text(face = axis.title.face, color = "black"),
+                      axis.title.y = ggplot2::element_text(face = axis.title.face, angle = 90, color = "black"),
+                      axis.text = ggplot2::element_text(face = axis.text.face, color = "black"),
                       axis.line = ggplot2::element_line(color = "black"),
                       axis.ticks = ggplot2::element_line(color = "black"),
                       plot.background = ggplot2::element_rect(fill = "white", color = "white"),
