@@ -17,7 +17,7 @@
     # Compute the grouped GO terms.
     out <- SCpubr::do_GroupedGOTermPlot(genes = genes.use,
                                         org.db = org.Hs.eg.db)
-  } else if (isFALSE(value)){
+  } else if (base::isFALSE(value)){
     message("This function can not be used without its suggested packages.")
     message("Check out which ones are needed using `SCpubr::state_dependencies()`.")
   }
